function [OUTPUT_nn,OUTPUT_bl,OUTPUT_bc,OUTPUT_DFTsincd]=RotateComparis_demo_CRC;

% Comparison of MatLab imrotate.m with its three interpolation methods
% (nearest neighb, bilinear and bicubic) and 3-step image rotation algorithm
% with discrete sinc-interpolation implemented in DFT and DCT domains.
% Compared are output images rotated by multiple of 360 degrees,
% computation time,
% rotation error found as difference between input and rotated images
% and rotation error DFT spectra.
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call [OUTPUT_nn,OUTPUT_bl,OUTPUT_bc,OUTPUT_DFTsinc]=RotateComparis_demo_CRC;

close all

m=menu('Select','Single rotation','Multiple rotations','10x36 degr. rotation of a random pattern');
switch m,
    case 1,
        load text256_175;INPUT=text256_175;
        fig_n(5);
        figure(1);
        image(INPUT);axis image;axis off;
        title('Input image','FontWeight','Bold');
        set(gca,'FontSize',8);
%         phi=round(360*rand);
        phi=inputdlg('Input rotation angle');
        phi=str2double(phi);

        t0=clock;
        OUTPUT_nn=imrotate(INPUT,-phi,'nearest','crop');
        T=etime(clock,t0);
        figure(2);image(OUTPUT_nn);axis image; axis off;
        title([num2str(phi),'grad-rotated image (n-neighbor); T=',num2str(T)],'FontWeight','Bold');
        set(gca,'FontSize',8);
    
        t0=clock;
        OUTPUT_bl=imrotate(INPUT,-phi,'bilinear','crop');
        T=etime(clock,t0);
        figure(3);image(OUTPUT_bl);axis image; axis off;
        title([num2str(phi),'grad-rotated image (bilinear); T=',num2str(T)],'FontWeight','Bold');
        set(gca,'FontSize',8);
    
        t0=clock;
        OUTPUT_bc=imrotate(INPUT,-phi,'bicubic','crop');
        T=etime(clock,t0);
        figure(4);image(OUTPUT_bc);axis image; axis off;
        title([num2str(phi),'grad-rotated image (bicubic); T=',num2str(T)],'FontWeight','Bold');
        set(gca,'FontSize',8);
    
        t0=clock;
        OUTPUT_DFTsincd=rotation_dft(INPUT,phi,0);
        T=etime(clock,t0);
        figure(5);image(OUTPUT_DFTsincd.*(OUTPUT_bc>0));axis image; axis off;
        title([num2str(phi),'grad-rotated image (DFT-sinc); T=',num2str(T)],'FontWeight','Bold');
        set(gca,'FontSize',8);
    
    case 2,
%         
        load text256_175;INPUT=text256_175;
        Phi=menu('Input total rotation angle','360','2*360','3*360','4*360','5*360');
        Phi=Phi*360;
        phi=menu('Input number of rotation steps','10','12','15','18','20');
        RotSt=[10 12 15 18 20];
        phi=360/RotSt(phi);
        N=Phi/phi;
        fig_n(5);
        figure(1);image(INPUT);axis image;axis off;title('Initial image','FontWeight','Bold')
        set(gca,'FontSize',8);
    %---------------------------------------------
        OUTPUT_nn=INPUT;
        T = 0;    
        for k=1:N,
            t0 = clock;
            OUTPUT_nn=imrotate(OUTPUT_nn,-phi,'nearest','crop');
            dT=etime(clock,t0);
            T=T+dT;
            figure(2);image(OUTPUT_nn);axis image; axis off;
            title([num2str(Phi),'gr. rotated image in ',num2str(N),'steps (n-neighb; T=',num2str(T),')'],'FontWeight','Bold');
            set(gca,'FontSize',8);
            drawnow
        end
    %------------------------------------------------------
        OUTPUT_bl=INPUT;
        T=0;
        for k=1:N,
            t0 = clock;
            OUTPUT_bl=imrotate(OUTPUT_bl,-phi,'bilinear','crop');
            dT=etime(clock,t0);
            T=T+dT;
            figure(3);image(OUTPUT_bl);axis image; axis off;
            title([num2str(Phi),'gr. rotated image in ',num2str(N),'steps (bilinear; T=',num2str(T),')'],'FontWeight','Bold');
            set(gca,'FontSize',8);
            drawnow
        end
    %------------------------------------------------------
        OUTPUT_bc=INPUT;
        T=0;
        for k=1:N,
            t0=clock;
            OUTPUT_bc=imrotate(OUTPUT_bc,-phi,'bicubic','crop');
            dT=etime(clock,t0);
            T=T+dT;
            figure(4);image(OUTPUT_bc);axis image; axis off;
            title([num2str(Phi),'gr. rotated image in ',num2str(N),'steps (bicubic; T=',num2str(T),')'],'FontWeight','Bold');
            set(gca,'FontSize',8);
            drawnow
        end
	%----------------------------------------------------------------
        OUTPUT_DFTsincd=INPUT;
        T=0;
        for k=1:N,
            t0=clock;
            OUTPUT_DFTsincd=rotation_dft(OUTPUT_DFTsincd,phi,0);
            dT=etime(clock,t0);
            T=T+dT;
            figure(5);image(OUTPUT_DFTsincd.*(OUTPUT_bc>0));axis image; axis off;
            title([num2str(Phi),'gr. rotated image in ',num2str(N),'steps (DFTsinc; T=',num2str(T),')'],'FontWeight','Bold');
            set(gca,'FontSize',8);
            drawnow
        end
    %------------------------------------------------------------------        
        mn=menu('Continue?','Yes','No');
        switch mn
            case 1
                [SzX,SzY]=size(INPUT);
                SzX=fix(SzX/4); SzY=fix(SzY/4);
        
                ERR_nn=INPUT(SzX:3*SzX-1,SzY:3*SzY-1)-OUTPUT_nn(SzX:3*SzX-1,SzY:3*SzY-1);
                [mn,StDev_nn]=std_2d(ERR_nn);
                sp_nn=fftshift(abs(fft2(ERR_nn)));
                sp_nn_sm=conv2(sp_nn,ones(3)/9,'same');
        
                ERR_bl=INPUT(SzX:3*SzX-1,SzY:3*SzY-1)-OUTPUT_bl(SzX:3*SzX-1,SzY:3*SzY-1);
                [mn,StDev_bl]=std_2d(ERR_bl);
                sp_bl=fftshift(abs(fft2(ERR_bl)));
                sp_bl_sm=conv2(sp_bl,ones(3)/9,'same');
        
                ERR_bc=INPUT(SzX:3*SzX-1,SzY:3*SzY-1)-OUTPUT_bc(SzX:3*SzX-1,SzY:3*SzY-1);
                [mn,StDev_bc]=std_2d(ERR_bc);
                sp_bc=fftshift(abs(fft2(ERR_bc)));
                sp_bc_sm=conv2(sp_bc,ones(3)/9,'same');
        
                ERR_sinc=INPUT(SzX:3*SzX-1,SzY:3*SzY-1)-OUTPUT_DFTsincd(SzX:3*SzX-1,SzY:3*SzY-1);
                [mn,StDev_sinc]=std_2d(ERR_sinc);
                sp_sinc=fftshift(abs(fft2(ERR_sinc)));
                sp_sinc_sm=conv2(sp_sinc,ones(3)/9,'same');
                s=1;
                while s==1,
                    mm=menu('Select','Display rotation error','Display rotation error spectra');
                    switch mm,
                        case 1;
                            fig_n(4);
                            nn_max=max(ERR_nn(:));
                            figure(1);image(255*ERR_nn/nn_max);axis image;axis off;
                            title(['Rot. error (n-neighbor)','FontWeight','Bold'],'FontWeight','Bold');
                            set(gca,'FontSize',8);
                            figure(2);image(255*ERR_bl/nn_max);axis image;axis off;
                            title('Rot. error (bilinear)','FontWeight','Bold');
                            set(gca,'FontSize',8);
                            figure(3);image(255*ERR_bc/nn_max);axis image;axis off;
                            title('Rot. error (bicubic)','FontWeight','Bold');
                            set(gca,'FontSize',8);
                            figure(4);image(255*ERR_sinc/nn_max);axis image;axis off;
                            title('Rot. error (DFTsinc)','FontWeight','Bold');
                            set(gca,'FontSize',8);
                        case 2
                            fig_n(5);
                            max_nn=max(sp_nn(:));
                            
                            figure(1);image(255*(sp_nn/max_nn));axis image;axis off;
                            colormap(jet);%colorbar('vert');
                            title('Rot. error spectrum (n-neighbor)','FontWeight','Bold');
                            set(gca,'FontSize',8);
            
                            figure(2);image(255*(sp_bl/max_nn));axis image;axis off;
                            colormap(jet);%colorbar('vert');
                            title('Rot. error spectrum (bilinear)','FontWeight','Bold');
                            set(gca,'FontSize',8);
            
                            figure(3);image(255*(sp_bc/max_nn));axis image;axis off;
                            colormap(jet);%colorbar('vert');
                            title('Rot. error spectrum (bicubic)','FontWeight','Bold');
                            set(gca,'FontSize',8);
            
                            figure(4);image(255*(sp_sinc/max_nn));axis image;axis off;
                            colormap(jet);%colorbar('vert');
                            title('Rot. error spectrum (DFTsinc)','FontWeight','Bold');
                            set(gca,'FontSize',8);
            
                            figure(5);
                            X=65:128;x=(1:64)/128;
                            mm=max(sp_nn_sm(64,X));
                            plot(x,sp_nn_sm(64,X)/mm,'r',x,sp_nn_sm(64,X)/mm,'r.',....
                                x,sp_bl_sm(64,X)/mm,'g',x,sp_bl_sm(64,X)/mm,'g.',.....
                                x,sp_bc_sm(64,X)/mm,'m',x,sp_bc_sm(64,X)/mm,'m.',.....
                                x,sp_sinc_sm(64,X)/mm,'b',x,sp_sinc_sm(64,X)/mm,'b.');axis tight;grid;
                            xlabel('Normalized frequency');
                            title(sprintf('1D sections (horiz) \n of the rotation error spectra (3x3 smoothed)'),'FontWeight','Bold')
                            legend('Interpolated n-neighb','Samples of nn',....
                            'Interpolated blinear','Samples of blinear',....
                            'Interpolated bcubic','Samples of bcubic',....
                            'Interpolated DFTsinc','Samples of DFTsincd');
                        end
                    s=menu('Continue?','Yes','No');  
                end
            case 2  %  
        end
    case 3
        sp_nn_sm=zeros(128); sp_bl_sm=zeros(128);sp_bc_sm=zeros(128);sp_sinc_sm=zeros(128);sp_dct_sm=zeros(128);
        t0=clock;
        fig_n(6);
        Nit=inputdlg('Input the number of iterations');Nit=str2double(Nit);
        for n=1:Nit;     
            INPUT=rand(256);sp_in=fft2(INPUT);INPUT=real(ifft2(sp_in./abs(sp_in)));
            INPUT=low_pass(INPUT,0.7,0.7,1,0,0);
            figure(1);
            display_1(INPUT);
            title([num2str(n),'-th realization of test random pattern'],'FontWeight','Bold','FontWeight','Bold');drawnow
            figure(2);
            display_1(abs(fftshift(fft2(INPUT))));
            title('Spectrum of 0.7x0.7-band-limited test pattern','FontWeight','Bold','FontWeight','Bold')
    %---------------------------------------------
            OUTPUT_nn=INPUT;    
            for k=1:10,
                OUTPUT_nn=imrotate(OUTPUT_nn,-36,'nearest','crop');
            end
    %------------------------------------------------------
            OUTPUT_bl=INPUT;
            for k=1:10,
                OUTPUT_bl=imrotate(OUTPUT_bl,-36,'bilinear','crop');
            end
    %------------------------------------------------------
            OUTPUT_bc=INPUT;
            for k=1:10,
                OUTPUT_bc=imrotate(OUTPUT_bc,-36,'bicubic','crop');
            end
	%----------------------------------------------------------------
            OUTPUT_DFTsincd=INPUT;
            for k=1:10,
%                 OUTPUT_DFTsincd=translt(OUTPUT_DFTsincd,1,1,36);
                OUTPUT_DFTsincd=rotation_dft(OUTPUT_DFTsincd,36,0);
            end
            Xx=129-64:129+63;Yy=129-64:129+63;
            
            ERR_nn=INPUT(Xx,Yy)-OUTPUT_nn(Xx,Yy);
            sp_nn_sm=sp_nn_sm+fftshift(abs(fft2(ERR_nn)).^2);
        
            ERR_bl=INPUT(Xx,Yy)-OUTPUT_bl(Xx,Yy);
            sp_bl_sm=sp_bl_sm+fftshift(abs(fft2(ERR_bl)).^2);
        
            ERR_bc=INPUT(Xx,Yy)-OUTPUT_bc(Xx,Yy);
            sp_bc_sm=sp_bc_sm+fftshift(abs(fft2(ERR_bc)).^2);
        
            ERR_sinc=INPUT(Xx,Yy)-OUTPUT_DFTsincd(Xx,Yy);
            sp_sinc_sm=sp_sinc_sm+fftshift(abs(fft2(ERR_sinc)).^2);    
        end
%         close(h);
%     fig_n(5);
%     figure(1);
    figure(3);
    display_1(sp_nn_sm);colormap(jet);
    title('Accumulated error spectrum: n-neighb','FontWeight','Bold','FontWeight','Bold');drawnow
%     figure(2);
    figure(4);
    display_1(sp_bl_sm);colormap(jet);
    title('Accumulated error spectrum: bilinear','FontWeight','Bold');drawnow
%     figure(3);
    figure(5)
    display_1(sp_bc_sm);colormap(jet);
    title('Accumulated error spectrum: bicubic','FontWeight','Bold');drawnow
%     figure(4);
    figure(6);
    display_1(sp_sinc_sm);colormap(jet);
    title('Accumulated error spectrum: DFTsinc','FontWeight','Bold');drawnow
end

%--------------------------------------------------------------------
function OUTPUT=rotation_dft(INPUT,phi,Displ_Flag);

% 3-step image rotation by shearing with discrete sinc-interpolation
% phi is the rotation angle in grad.
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call OUTPUT=myrotate_demo(INPUT,phi,Displ_Flag);

[SzX SzY] =size(INPUT);
phi=pi*phi/180;
OUTPUT=INPUT;
if Displ_Flag==1, figure(1);
   image(OUTPUT);title('Initial image','FontWeight','Bold');axis image;axis off;drawnow;
end
%FIRST PASS
u=(1-cos(phi))/sin(phi);
for x=1:SzX,
Ux=u*(x-SzX/2-1/2);
My=zeros(1,SzY);
r=1:SzY/2+1; Wn=i*2*pi*Ux/SzY;
My(r)=exp(Wn*(r-1)); My(SzY:-1:SzY/2+2)=conj(My(2:SzY/2));
		spinput=fftnorm(INPUT(x,:));
		spinput=spinput.*My;
OUTPUT(x,:)=real(ifftnorm([spinput(1:SzY/2) real(spinput(SzY/2+1)) spinput(SzY/2+2:SzY)]));
end
if Displ_Flag==1, figure(2);
	image(OUTPUT);title('First pass','FontWeight','Bold');axis image;axis off;
	drawnow
end

% SECOND PASS
u=-sin(phi);
for y=1:SzY,
Uy=u*(y-SzY/2-1/2);
Mx=zeros(SzY,1);
r=1:SzX/2+1; Wn=i*2*pi*Uy/SzX;
Mx(r)=exp(Wn*(r-1)); Mx(SzX:-1:SzX/2+2)=conj(Mx(2:SzX/2));
		spinput=fftnorm(OUTPUT(:,y));
		spinput=spinput.*Mx;
OUTPUT(:,y)=real(ifftnorm([spinput(1:SzX/2);real(spinput(SzX/2+1));spinput(SzX/2+2:SzX)]));
end

if Displ_Flag==1,figure(3);
   image(OUTPUT);title('Second pass','FontWeight','Bold');axis image;axis off;
   drawnow
end

%THIRD PASS
u=(1-cos(phi))/sin(phi);
for x=1:SzX,
Ux=u*(x-SzX/2-1/2);
My=zeros(1,SzY);
r=1:SzY/2+1; Wn=i*2*pi*Ux/SzY;
My(r)=exp(Wn*(r-1)); My(SzY:-1:SzY/2+2)=conj(My(2:SzY/2));
		spinput=fftnorm(OUTPUT(x,:));
		spinput=spinput.*My;
OUTPUT(x,:)=real(ifftnorm([spinput(1:SzY/2) real(spinput(SzY/2+1)) spinput(SzY/2+2:SzY)]));
end

%OUTPUT=(OUTPUT>=0).*OUTPUT;
%output=(OUTPUT<=255).*OUTPUT+255*ones(size(OUTPUT)).*(OUTPUT>255);
if Displ_Flag==1,
   figure(4);
	image(OUTPUT);title('Third pass: rotated image','FontWeight','Bold');axis image;axis off;
   drawnow
end

%-----------------------------------------------------------------
function fig_n(N)
% Generating N square figure windows of ScreenVerSz/2*ScreenVerSz pixels
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)
% Version July 15, 2001
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call figure_n(N)

% Generating a figure window
ScrSz=get(0,'ScreenSize');
HorSz=(ScrSz(3)-20); VerSz=(ScrSz(4));
close all;

switch N;
case 1, 
   Nvert=1;Nhor=1;H_Sz=(HorSz-80); V_Sz=(VerSz-120);
   figure('Position',[30 VerSz-30-V_Sz V_Sz V_Sz]); colormap(gray(256))
case 2, 
   Nvert=1;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-120);
   figure('Position',[30 VerSz-30-H_Sz H_Sz H_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-H_Sz H_Sz H_Sz]);colormap(gray(256))
case 3, 
   Nvert=2;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
case 4, 
   Nvert=2;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
case 5,
   Nvert=2;Nhor=3;H_Sz=(HorSz-80)/3; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+2*H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
case 6, 
   Nvert=2;Nhor=3;H_Sz=(HorSz-80)/3; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+2*H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+2*H_Sz VerSz-75-2*V_Sz H_Sz V_Sz]);colormap(gray(256))
end
   
%------------------------------------------------------
function display_1(INPIMG)
% Displays a 2-D signal as an image 
% with normalization by gobal maximum and minimum
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il) 
% Call display_1(INPIMG)

colormap(gray(256))
Min=min(min(INPIMG)); Max=max(max(INPIMG));
if Max-Min==0, im=zeros(size(INPIMG)); else im=255*(INPIMG-Min)/(Max-Min); end

image(im);axis image;axis off;
%---------------------------------------------------------
function z=fftnorm(signal);

% Normalized direct 1-D FFT
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)
% Callz=fftnorm(signal);

N=sqrt(size(signal,2));
signal=signal/N;
z=fft(signal);
%--------------------------------------------------------------------------
function z=ifftnorm(signal);

% Normalized inverse 1D FFT
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)
% Call z=ifftnorm(signal);

N=sqrt(size(signal,2));
signal=signal*N;
z=ifft(signal);
%--------------------------------------------------------------------------
function [mn,v]=std_2d(INP);

% Standard deviation and mean of a 2-D array
% z=[mean stdeviation];
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call [mn v]=std_2d(INP)

[SzX SzY]=size(INP);
mn=sum(sum(INP))/(SzX*SzY);
v=sqrt(sum(sum((INP-mn).^2))/(SzX*SzY));
%-------------------------------------------------------------------------
function OUTIMG=load__img;

% A browser for loading images in all formats
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call OUTIMG=load__img;

dr=pwd;
% cd c:\yaro_data\images\;
[FILENAME, PATHNAME] = uigetfile('*.*', 'LOAD IMAGE in *.tif format',10,10);
eval(['cd ',dr]);
OUTIMG=imread([PATHNAME FILENAME]);
OUTIMG=double(OUTIMG);
%---------------------------------------------------------


