function [ERR1,ERR2,ERR_d1,ERR_d2]=differentiator_comparison_CRC(N,M,Ntest);

% Test of different differentiation methods with simulated continuous
% and sampled sinusoidal signal
% N - number of samples of signal and its derivative; recommended N=511;
% M - oversampling index; recommended M=32
% Ntest - number of test realizations; recommended Ntest=50;
% 
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il),
% Call  [ERR1,ERR2,ERR_d1,ERR_d2]=differentiator_comparison_CRC(N,M,Ntest);


fig_n(2);
NM=N*M;
k=0:2*NM-1;
% d1=[1 -1];
d1=[0.5 0 -0.5];
d2=[-1/12 2/3 0 -2/3 1/12];
ERR1=zeros(N,16);
ERR2=zeros(N,16);
ERR_d1=zeros(N,16);
ERR_d2=zeros(N,16);

ApodMask=apodmask(1,N,1,round(0.9*N));
m=menu('Select','No apod mask','Apodmask');
if m==1, txt='No ApodMask';else txt='ApodMask';end
for K=1:16,
    for n=1:Ntest,
        s=rand(1,2*NM); 
        sp=fft(s);
        s=real(ifft(sp./(abs(sp))+eps));
        [mn StDev]=std2d(s);
        s=(s-mn)/StDev;

% ------------------------------------------Low pass filter mask------------
        KN=round(K*N/32);
        lpf_mask=ones(1,KN+1);
        lpf_mask(KN+1)=0.5;
        lpf_mask=[lpf_mask,zeros(1,2*NM-2*KN-1),fliplr(lpf_mask(2:KN+1))];

        sp_s=fft(s);
        s_lpf=real(ifft(sp_s.*lpf_mask));

% -------------------------------------------Computing derivatives----------
        deriv_s_lpf=NM*differentiator_1D(s_lpf)/(2*pi);

        ss_lpf=s_lpf(round(NM/2)+1:M:round(3*NM/2));                  %--------------------Samples of low-pass filtered input
        if m==2, ss_lpf=ss_lpf.*ApodMask;end
        deriv_ss_lpf_id=deriv_s_lpf(round(NM/2)+1:M:round(3*NM/2));   %-------------Samples of ideal derivative of low-pass filtered input
        deriv_ss_lpf=N*differentiator_1D(ss_lpf)/(2*pi);

        [mn StDevDeriv]=std2d(deriv_ss_lpf_id);
% -------------------------------------Odd extension gives poor results!!!-------------
        sss_lpf_even=[ss_lpf,fliplr(ss_lpf)];

        deriv_sss_lpf_even=N*differentiator_1D(sss_lpf_even)/(2*pi);deriv_sss_lpf_even=deriv_sss_lpf_even(1:N);

        deriv_d1=N*conv2(ss_lpf,d1,'same')'/(2*pi);
        deriv_d2=N*conv2(ss_lpf,d2,'same')'/(2*pi);

% ----------------------------Results & graphs-----------------------------

        err1=(deriv_ss_lpf-deriv_ss_lpf_id)/StDevDeriv;
        err2=(deriv_sss_lpf_even-deriv_ss_lpf_id)/StDevDeriv;
        err_d1=(deriv_d1-deriv_ss_lpf_id)/StDevDeriv;
        err_d2=(deriv_d2-deriv_ss_lpf_id)/StDevDeriv;
        
        figure(1);
        kkk=1:N;
        plot(kkk,deriv_ss_lpf/K,'r',kkk,deriv_ss_lpf_id/K,'b',kkk,ss_lpf,'g','LineWidth',2);axis([1 N -1.5 1.5]);
        grid on;
        set(gca,'FontSize',10);set(gca,'FontName','Helvetica');set(gca,'FontWeight','Bold');
        legend('Derivative\_DFT','Derivative\_ideal','Signal')
        title(['Signal and its derivatives; Bandwidth=',num2str(K/16,3)]);
        xlabel('Sample index','FontWeight','Bold');
        drawnow
        ERR1(:,K)=ERR1(:,K)+(err1).^2;
        ERR2(:,K)=ERR2(:,K)+(err2).^2;
        ERR_d1(:,K)=ERR_d1(:,K)+(err_d1).^2;
        ERR_d2(:,K)=ERR_d2(:,K)+(err_d2).^2;

        figure(2)
        semilogy(kkk,sqrt(ERR1(kkk,K))/n,'r',kkk,sqrt(ERR2(kkk,K))/n,'b',kkk,sqrt(ERR_d1(kkk,K))/n,'g',kkk,sqrt(ERR_d2(kkk,K))/n,'m','LineWidth',2);
        axis tight;grid on
        set(gca,'FontSize',10);set(gca,'FontName','Helvetica');set(gca,'FontWeight','Bold');
        title(['Differentiation error, Bandwidth= ',num2str(K/16,3),'; ',num2str(n),'-th realization'],'FontWeight','Bold');
        xlabel('Sample index','FontWeight','Bold');
        legend('DFT','DCT','d1','d2');
        drawnow
    end
end

ERR1=sqrt(ERR1/Ntest);
ERR2=sqrt(ERR2/Ntest); 
ERR_d1=sqrt(ERR_d1/Ntest);
ERR_d2=sqrt(ERR_d2/Ntest);

fig_n(3);
figure(1)
for K=4:4:16,
    semilogy(kkk,ERR1(kkk,K),'r',kkk,ERR_d1(kkk,K),'m',kkk,ERR_d2(kkk,K),'b','LineWidth',2);axis tight;grid
    hold on
end
grid on
set(gca,'FontSize',10);set(gca,'FontName','Helvetica');set(gca,'FontWeight','Bold');
hold off
title(['Differentiation error,',num2str(n),' realizations; ',txt],'FontWeight','Bold');
xlabel('Sample index','FontWeight','Bold');
legend('DFT','d1','d2');

figure(2)
for K=4:4:16,
    semilogy(kkk,ERR1(kkk,K),'r',kkk,ERR2(kkk,K),'b',kkk,ERR_d2(kkk,K),'g','LineWidth',2);axis tight;grid
    hold on
end
grid on
set(gca,'FontSize',10);set(gca,'FontName','Helvetica');set(gca,'FontWeight','Bold');
hold off
title(['Differentiation error, ',num2str(n),' realizations; ',txt],'FontWeight','Bold');
xlabel('Sample index','FontWeight','Bold');
legend('DFT','DCT','d2');


figure(3)
BW=1:16;
X=fix(N/8:7*N/8);
semilogy(BW/16,mean(ERR_d1(X,:)),BW/16,mean(ERR_d2(X,:)),BW/16,mean(ERR1(X,:)),BW/16,mean(ERR2(X,:)),'LineWidth',2);
axis tight;grid on
set(gca,'FontSize',10);set(gca,'FontName','Helvetica');set(gca,'FontWeight','Bold');
xlabel('Signal bandwidth (in fraction of the base band)');
title(['Differentiation error StdDev over the signal central half; ',txt],'FontWeight','Bold')
legend('d1','d2','DFT','DCT');
%---------------------------------------------------------------------------------
function [mn,v]=std2d(INP);

% Standard deviation and mean of a 2-D array
% z=[mean stdeviation];
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call [mn v]=std2d(INP)

[SzX SzY]=size(INP);
mn=sum(sum(INP))/(SzX*SzY);
v=sqrt(sum(sum((INP-mn).^2))/(SzX*SzY));
%------------------------------------------------------------------------------------
%-----------------------------------------------------------------
function fig_n(N)
% Generating N square figure windows of ScreenVerSz/2*ScreenVerSz pixels
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)
% Version July 15, 2001
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call fig_n(N)

% Generating a figure window
ScrSz=get(0,'ScreenSize');
HorSz=(ScrSz(3)-20); VerSz=(ScrSz(4));
close all;

switch N;
case 1, 
   Nvert=1;Nhor=1;H_Sz=(HorSz-80); V_Sz=(VerSz-120);
   figure('Position',[30 VerSz-30-V_Sz V_Sz V_Sz]); colormap(gray(256))
case 2, 
   Nvert=1;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-120);
   figure('Position',[30 VerSz-30-H_Sz H_Sz H_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-H_Sz H_Sz H_Sz]);colormap(gray(256))
case 3, 
   Nvert=2;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
case 4, 
   Nvert=2;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
case 5,
   Nvert=2;Nhor=3;H_Sz=(HorSz-80)/3; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+2*H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
case 6, 
   Nvert=2;Nhor=3;H_Sz=(HorSz-80)/3; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+2*H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+2*H_Sz VerSz-75-2*V_Sz H_Sz V_Sz]);colormap(gray(256))
end
   
%------------------------------------------------------
function display_1(INPIMG)
% Displays a 2-D signal as an image 
% with normalization by gobal maximum and minimum
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il) 
% Call display_1(INPIMG)

colormap(gray(256))
Min=min(min(INPIMG)); Max=max(max(INPIMG));
if Max-Min==0, im=zeros(size(INPIMG)); else im=255*(INPIMG-Min)/(Max-Min); end

image(im);axis image;axis off;
%---------------------------------------------------------

function OUT=apodmask(SzX,SzY,WindowX,WindowY);

% Generating 1-D or 2-D tapered rectangular apodizing mask
% Szx,SzY - size of the mask; 
% WindowX, WondowY - size of its cental completely transparent section
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il). Sept. 25, 2002  
% Call OUT=apodmask(SzX,SzY,WindowX,WindowY);

OUTx=ones(SzX,1);
Lx=fix((SzX-WindowX)/2);
k=1:Lx;
OUTx(1:Lx)=(1+cos(2*pi*(1/2+(k-1)/(2*Lx))))/2;
OUTx(SzX-Lx+1:SzX)=(1+cos(2*pi*k/(2*Lx)))/2;

OUTy=ones(1,SzY);
Ly=fix((SzY-WindowY)/2);
k=1:Ly;
OUTy(1:Ly)=(1+cos(2*pi*(1/2+(k-1)/(2*Ly))))/2;
OUTy(SzY-Ly+1:SzY)=(1+cos(2*pi*k/(2*Ly)))/2;

OUT=kron(OUTx,OUTy);

